def test_sum():
    assert 6 == 6, "Should be 6"


def test_sum_tuple():
    assert 6 == 6, "Should be 6"


if __name__ == "__main__":
    test_sum()
    test_sum_tuple()
    print("Everything passed")
